Package: chikvdf-hw
Version: {{ INSTALLER_VERSION }}
Architecture: {{ PLATFORM }}
Maintainer: Chik Network Inc <hello@chiknetwork.com>
Description: Hardware VDF Client
Depends: libgmp-dev, libboost-system-dev
